package org.xesstandard.extensions;

import java.net.URI;

import org.xesstandard.model.XesAttribute;
import org.xesstandard.model.XesComponent;
import org.xesstandard.model.attributes.XesStringAttribute;
import org.xesstandard.model.attributes.impl.XesStringAttributeImpl;
import org.xesstandard.model.impl.XesExtensionImpl;

public class XesSemanticExtension extends XesExtensionImpl {

	public final static String MODELREFERENCE = "semantic:modelReference";

	private final static XesSemanticExtension singleton = new XesSemanticExtension();

	private XesSemanticExtension() {
		super(true);
		setName("Semantic");
		setPrefix("semantic");
		try {
			setURI(new URI("http://www.xes-standard.org/semantic.xesext"));
		} catch (Exception e) {
			System.err.println(e.getMessage());
		}

		XesAttribute attribute = new XesStringAttributeImpl();
		attribute.setKey(MODELREFERENCE);
		addLogAttribute(attribute);
		addTraceAttribute(attribute);
		addEventAttribute(attribute);
		addMetaAttribute(attribute);
		addAlias(attribute, "EN", "Ontology Model Reference");
		addAlias(attribute, "DE", "Ontologie-Modellreferenz");
		addAlias(attribute, "FR", "R\u00e9f\u00e9rence au Mod\u00e8le Ontologique");
		addAlias(attribute, "ES", "Referencia de Modelo Ontol\u00f3gico");
		addAlias(attribute, "PT", "Refer\u00eancia de Modelo Ontol\u00f3gico");
	}

	/**
	 * @return The single instance of this extension.
	 */
	public static XesSemanticExtension instance() {
		return singleton;
	}

	private XesStringAttribute setString(XesComponent component, String key, String value) {
		XesStringAttribute attribute = new XesStringAttributeImpl();
		attribute.setKey(key);
		attribute.setValue(value);
		component.addAttribute(attribute);
		return attribute;
	}
	
	/**
	 * Returns the model reference of the given component, that is, the value of
	 * the attribute with key "semantic:modelReference".
	 * 
	 * @param component
	 *            The given log, trace, event, or attribute.
	 * @return The model reference of the given component.
	 */
	public String getModelReference(XesComponent component) {
		return component.getString(MODELREFERENCE);
	}

	/**
	 * Sets the model reference for the given component, that is, the value of
	 * the attribute with key "semantic:modelReference".
	 * 
	 * @param component
	 *            The given log, trace, event, or attribute.
	 * @param value
	 *            The model reference to set.
	 * @return The attribute containing the model reference.
	 */
	public XesStringAttribute setModelReference(XesComponent component, String value) {
		return setString(component, MODELREFERENCE, value);
	}
}
