package org.xesstandard.model;

import java.net.URI;
import java.util.ArrayList;
import java.util.List;

import org.xesstandard.extensions.XesArtifactLifecycleExtension;
import org.xesstandard.extensions.XesConceptExtension;
import org.xesstandard.extensions.XesCostExtension;
import org.xesstandard.extensions.XesIdExtension;
import org.xesstandard.extensions.XesLifecycleExtension;
import org.xesstandard.extensions.XesMicroExtension;
import org.xesstandard.extensions.XesOrganizationalExtension;
import org.xesstandard.extensions.XesSemanticExtension;
import org.xesstandard.extensions.XesSoftwareCommunicationExtension;
import org.xesstandard.extensions.XesSoftwareEventExtension;
import org.xesstandard.extensions.XesSoftwareTelemetryExtension;
import org.xesstandard.extensions.XesTimeExtension;

public class XesExtensionManager {

	private static XesExtensionManager singleton = new XesExtensionManager();

	private List<XesExtension> extensions;

	private XesExtensionManager() {
		extensions = new ArrayList<XesExtension>();
		extensions.add(XesConceptExtension.instance());
		extensions.add(XesLifecycleExtension.instance());
		extensions.add(XesOrganizationalExtension.instance());
		extensions.add(XesTimeExtension.instance());
		extensions.add(XesSemanticExtension.instance());
		extensions.add(XesIdExtension.instance());
		extensions.add(XesCostExtension.instance());
		extensions.add(XesMicroExtension.instance());
		extensions.add(XesSoftwareEventExtension.instance());
		extensions.add(XesSoftwareCommunicationExtension.instance());
		extensions.add(XesSoftwareTelemetryExtension.instance());
		extensions.add(XesArtifactLifecycleExtension.instance());
	}

	/**
	 * 
	 * @return The unique instance of the XES extension manager.
	 */
	public static XesExtensionManager instance() {
		return singleton;
	}

	/**
	 * Adds the given extension to the list of known extensions.
	 * 
	 * @param extension
	 *            The given extension.
	 */
	public void addExtension(XesExtension extension) {
		extensions.add(extension);
	}

	/**
	 * Searches for the extension with given name (may be <null>), prefix (may
	 * be <null>), and URI (may be <null>). If all three are null, the first extension
	 * in the list is returned.
	 * 
	 * @param name
	 *            The given name. If <null>, the name is ignored in the search.
	 * @param prefix
	 *            The given prefix. If <null>, the prefix is ignored in the
	 *            search.
	 * @param uri
	 *            The given URI. If <null>, the URI is ignored in the search.
	 * @return The first extension with given name (if not <null>), given prefix
	 *         (if not <null>), and given uri (if not <null>).
	 */
	public XesExtension getExtension(String name, String prefix, URI uri) {
		for (XesExtension extension : extensions) {
			if ((name == null || name.equals(extension.getName()))
					&& (prefix == null || prefix.equals(extension.getPrefix()))
					&& (uri == null || uri.equals(extension.getURI()))) {
				return extension;
			}
		}
		return null;
	}
}
