package org.xesstandard.model.impl;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.UUID;

import org.xesstandard.model.XesAttribute;
import org.xesstandard.model.XesAttributeType;
import org.xesstandard.model.XesComponent;
import org.xesstandard.model.attributes.XesBooleanAttribute;
import org.xesstandard.model.attributes.XesDateTimeAttribute;
import org.xesstandard.model.attributes.XesIdAttribute;
import org.xesstandard.model.attributes.XesIntegerNumberAttribute;
import org.xesstandard.model.attributes.XesListAttribute;
import org.xesstandard.model.attributes.XesRealNumberAttribute;
import org.xesstandard.model.attributes.XesStringAttribute;

public abstract class XesComponentImpl implements XesComponent {

	/*
	 * A map from keys to sub attributes.
	 */
	private Map<String, XesAttribute> attributes = new HashMap<String, XesAttribute>();
	
	/*
	 * (non-Javadoc)
	 * @see org.xesstandard.model.XesComponent#addAttribute(org.xesstandard.model.XesAttribute)
	 */
	public void addAttribute(XesAttribute attribute) {
		attributes.put(attribute.getKey(), attribute);
	}

	/*
	 * (non-Javadoc)
	 * @see org.xesstandard.model.XesComponent#getAttributes()
	 */
	public List<XesAttribute> getAttributes() {
		return new ArrayList<XesAttribute>(attributes.values());
	}
	
	/*
	 * (non-Javadoc)
	 * @see org.xesstandard.model.XesComponent#getStringValue(java.lang.String)
	 */
	public String getStringValue(String key) {
		XesAttribute attribute = attributes.get(key);
		if (attribute == null) {
			return null;
		}
		return attribute.getStringValue();
	}

	/*
	 * (non-Javadoc)
	 * @see org.xesstandard.model.XesComponent#getType(java.lang.String)
	 */
	public XesAttributeType getType(String key) {
		XesAttribute attribute = attributes.get(key);
		if (attribute == null) {
			// No such attribute. Return default type.
			return XesAttributeType.NONE;
		}
		return attribute.getType();
	}
	
	/*
	 * (non-Javadoc)
	 * @see org.xesstandard.model.XesComponent#getString(java.lang.String)
	 */
	public String getString(String key) {
		XesAttribute attribute = attributes.get(key);
		if (attribute != null && attribute.getType() == XesAttributeType.STRING) {
			return ((XesStringAttribute) attribute).getValue();
		}
		// Not a string attribute. Return <null>.
		return null;
	}
	
	/*
	 * (non-Javadoc)
	 * @see org.xesstandard.model.XesComponent#getDateTime(java.lang.String)
	 */
	public Date getDateTime(String key) {
		XesAttribute attribute = attributes.get(key);
		if (attribute != null && attribute.getType() == XesAttributeType.DATETIME) {
			return ((XesDateTimeAttribute) attribute).getValue();
		}
		// Not a date attribute. Return <null>.
		return null;
	}
	
	/*
	 * (non-Javadoc)
	 * @see org.xesstandard.model.XesComponent#getId(java.lang.String)
	 */
	public UUID getId(String key) {
		XesAttribute attribute = attributes.get(key);
		if (attribute != null && attribute.getType() == XesAttributeType.ID) {
			return ((XesIdAttribute) attribute).getValue();
		}
		// Not a id attribute. Return <null>.
		return null;
	}
	
	/*
	 * (non-Javadoc)
	 * @see org.xesstandard.model.XesComponent#getIntegerNumber(java.lang.String)
	 */
	public Long getIntegerNumber(String key) {
		XesAttribute attribute = attributes.get(key);
		if (attribute != null && attribute.getType() == XesAttributeType.INTEGERNUMBER) {
			return ((XesIntegerNumberAttribute) attribute).getValue();
		}
		// Not an integer number attribute. Return <null>.
		return null;
	}
	
	/*
	 * (non-Javadoc)
	 * @see org.xesstandard.model.XesComponent#getRealNumber(java.lang.String)
	 */
	public Double getRealNumber(String key) {
		XesAttribute attribute = attributes.get(key);
		if (attribute != null && attribute.getType() == XesAttributeType.REALNUMBER) {
			return ((XesRealNumberAttribute) attribute).getValue();
		}
		// Not a real number attribute. Return <null>.
		return null;
	}
	
	/*
	 * (non-Javadoc)
	 * @see org.xesstandard.model.XesComponent#getBoolean(java.lang.String)
	 */
	public Boolean getBoolean(String key) {
		XesAttribute attribute = attributes.get(key);
		if (attribute != null && attribute.getType() == XesAttributeType.BOOLEAN) {
			return ((XesBooleanAttribute) attribute).getValue();
		}
		// Not a boolean attribute. Return <null>.
		return null;
	}
	
	/*
	 * (non-Javadoc)
	 * @see org.xesstandard.model.XesComponent#getList(java.lang.String)
	 */
	public List<XesAttribute> getList(String key) {
		XesAttribute attribute = attributes.get(key);
		if (attribute != null && attribute.getType() == XesAttributeType.LIST) {
			return ((XesListAttribute) attribute).getValues();
		}
		// Not a list attribute. Return <null>.
		return null;
	}
	
	/*
	 * (non-Javadoc)
	 * @see org.xesstandard.model.XesComponent#setString(java.lang.String, java.lang.String)
	 */
	public String setString(String key, String value) {
		XesAttribute attribute = attributes.get(key);
		if (attribute.getType() == XesAttributeType.STRING) {
			String oldValue = ((XesStringAttribute) attribute).getValue();
			((XesStringAttribute) attribute).setValue(value);
			return oldValue;
		}
		return null;
	}
	
	/*
	 * (non-Javadoc)
	 * @see org.xesstandard.model.XesComponent#setDateTime(java.lang.String, java.util.Date)
	 */
	public Date setDateTime(String key, Date value) {
		XesAttribute attribute = attributes.get(key);
		if (attribute.getType() == XesAttributeType.DATETIME) {
			Date oldValue = ((XesDateTimeAttribute) attribute).getValue();
			((XesDateTimeAttribute) attribute).setValue(value);
			return oldValue;
		}
		return null;
	}
	
	/*
	 * (non-Javadoc)
	 * @see org.xesstandard.model.XesComponent#setId(java.lang.String, java.util.UUID)
	 */
	public UUID setId(String key, UUID value) {
		XesAttribute attribute = attributes.get(key);
		if (attribute.getType() == XesAttributeType.ID) {
			UUID oldValue = ((XesIdAttribute) attribute).getValue();
			((XesIdAttribute) attribute).setValue(value);
			return oldValue;
		}
		return null;
	}
	
	/*
	 * (non-Javadoc)
	 * @see org.xesstandard.model.XesComponent#setIntegerNumber(java.lang.String, java.lang.Long)
	 */
	public Long setIntegerNumber(String key, Long value) {
		XesAttribute attribute = attributes.get(key);
		if (attribute.getType() == XesAttributeType.INTEGERNUMBER) {
			Long oldValue = ((XesIntegerNumberAttribute) attribute).getValue();
			((XesIntegerNumberAttribute) attribute).setValue(value);
			return oldValue;
		}
		return null;
	}
	
	/*
	 * (non-Javadoc)
	 * @see org.xesstandard.model.XesComponent#setRealNumber(java.lang.String, java.lang.Double)
	 */
	public Double setRealNumber(String key, Double value) {
		XesAttribute attribute = attributes.get(key);
		if (attribute.getType() == XesAttributeType.REALNUMBER) {
			Double oldValue = ((XesRealNumberAttribute) attribute).getValue();
			((XesRealNumberAttribute) attribute).setValue(value);
			return oldValue;
		}
		return null;
	}
	
	/*
	 * (non-Javadoc)
	 * @see org.xesstandard.model.XesComponent#setBoolean(java.lang.String, java.lang.Boolean)
	 */
	public Boolean setBoolean(String key, Boolean value) {
		XesAttribute attribute = attributes.get(key);
		if (attribute.getType() == XesAttributeType.BOOLEAN) {
			Boolean oldValue = ((XesBooleanAttribute) attribute).getValue();
			((XesBooleanAttribute) attribute).setValue(value);
			return oldValue;
		}
		return null;
	}
	
	/*
	 * (non-Javadoc)
	 * @see org.xesstandard.model.XesComponent#setList(java.lang.String, java.util.List)
	 */
	public List<XesAttribute> setList(String key, List<XesAttribute> values) {
		XesAttribute attribute = attributes.get(key);
		if (attribute.getType() == XesAttributeType.LIST) {
			List<XesAttribute> oldValues = ((XesListAttribute) attribute).getValues();
			((XesListAttribute) attribute).setValues(values);
			return oldValues;
		}
		return null;
	}
	
	public String toString() {
		if (attributes.isEmpty()) {
			return "()";
		}
		if (attributes.keySet().contains("concept:name")) {
			return getStringValue("concept:name");
		}
		StringBuffer buf = new StringBuffer();
		String div = "(";
		for (XesAttribute attribute: attributes.values()) {
			buf.append(div);
			buf.append(attribute);
			div = ",";
		}
		buf.append(")");
		return buf.toString();
	}
}
