package org.xesstandard.xml.issues;

import org.xesstandard.model.XesAttribute;
import org.xesstandard.model.XesClassifier;
import org.xesstandard.model.XesComponent;
import org.xesstandard.model.XesEvent;
import org.xesstandard.model.XesExtension;
import org.xesstandard.model.XesGlobal;
import org.xesstandard.model.XesTrace;
import org.xesstandard.model.XesValues;
import org.xesstandard.xml.XesXmlIssueLevel;

public class XesXmlIssueParent extends XesAbstractXmlIssue {

	/**
	 * Issue raised when an element (child) appears in the wrong container (parent).
	 */
	private Object object;
	private XesComponent parent;
	String label;
	String parents;
	
	public XesXmlIssueParent(XesTrace trace, XesComponent parent) {
		super(XesXmlIssueLevel.CONFORMING);
		this.object = trace;
		this.parent = parent;
		label = "Trace";
		parents = "a log";
	}
	
	public XesXmlIssueParent(XesEvent event, XesComponent parent) {
		super(XesXmlIssueLevel.CONFORMING);
		this.object = event;
		this.parent = parent;
		label = "Event";
		parents = "a log or a trace";
	}
	
	public XesXmlIssueParent(XesAttribute attribute, XesComponent parent) {
		super(XesXmlIssueLevel.CONFORMING);
		this.object = attribute;
		this.parent = parent;
		label = "Attribute";
		parents = "a component, a global, or a value for a list";
	}
	
	public XesXmlIssueParent(XesExtension extension, XesComponent parent) {
		super(XesXmlIssueLevel.CONFORMING);
		this.object = extension;
		this.parent = parent;
		label = "Extension";
		parents = "a log";
	}
	
	public XesXmlIssueParent(XesGlobal global, XesComponent parent) {
		super(XesXmlIssueLevel.CONFORMING);
		this.object = global;
		this.parent = parent;
		label = "Global";
		parents = "a log";
	}
	
	public XesXmlIssueParent(XesClassifier classifier, XesComponent parent) {
		super(XesXmlIssueLevel.CONFORMING);
		this.object = classifier;
		this.parent = parent;
		label = "Classifier";
		parents = "a log";
	}
	
	public XesXmlIssueParent(XesValues values, XesComponent parent) {
		super(XesXmlIssueLevel.CONFORMING);
		this.object = values;
		this.parent = parent;
		label = "Values";
		parents = "a list";
	}
	
	public String toString() {
		return super.toString() + label + " is not child element of " + parents + " (see Figure 2).";
	}
}
